#!/usr/bin/perl
if(@ARGV != 4) {
	print "Usage: perl sr.pl configFile seqFile pirsfId outputGffFile\n";
	exit 1;
}

my $conf = @ARGV[0];
my $querySeqFile  = @ARGV[1];
my $pirsf = uc(@ARGV[2]);
my $gffFile = @ARGV[3];

my $debug = 0;
my $tmpDir= "";
my $hmmSearchCmd = "";
my $hmmAlignCmd = "";
my $srHmmDir = "";
my $srTpSeq = "";

my @srIds = ();

checkConfFile();

my %templateSeqs = getTemplateSeqs($srTpSeq);

prepareOutDir($tmpDir, $pirsf);

my %gff = ();

my @srIds = sort(getSRIds($srHmmDir, $pirsf));
foreach(@srIds) {
	my $sr = $_;
	if($debug == 1) {
		print "\n".$sr."\n";
	}
	$srHmm = $srHmmDir."/".$sr.".hmm";
	$template = $templateSeqs{$sr};
	if(-e $srHmm && -e $querySeqFile && $template) {    
		runHmmer($tmpDir, $pirsf, $sr, $srHmm, $template, $querySeqFile);		
	}
}
if($debug == 0) {
	system("rm -rf $tmpDir");
}	


sub runHmmer {
	($tmpDir, $pirsf, $sr, $srHmm, $templateSeq, $querySeqFile) = @_;
	@template = split(/\n/, $templateSeq);
	$templateAC = "";
	foreach(@template) {
		if($_ =~ /^>/) {
			($templateAC) = (split(/\t/, $_))[0];
			$templateAC =~ s/^>//;
		}	
	}
	$querySeq = getQuerySeq($querySeqFile);
	
	$queryFile = $tmpDir."/".$pirsf."/hmm/seq/".$sr.".query.seq";
	$seqFile = $tmpDir."/".$pirsf."/hmm/seq/".$sr.".seq";
	$searchOutFile = $tmpDir."/".$pirsf."/hmm/search/".$sr.".out";
	$alnOutFile = $tmpDir."/".$pirsf."/hmm/aln/".$sr.".aln";
	$alnMsgFile = $tmpDir."/".$pirsf."/hmm/aln/".$sr.".msg";
	open(QUERY, ">", $queryFile) or die "Can't open $queryFile\n";
	print QUERY $querySeq;	
	close(QUERY); 
	open(SEQ, ">", $seqFile) or die "Can't open $seqFile\n";
	print SEQ $templateSeq;
	print SEQ $querySeq;	
	close(SEQ); 
	$search = "$hmmSearchCmd $srHmm $queryFile > $searchOutFile";
	$aln = "$hmmAlignCmd -o $alnOutFile $srHmm $seqFile > $alnMsgFile";
	if($debug == 1) {
		print $search."\n";
	}
	system($search);
	if($debug == 1) {
		print $aln."\n\n";
	}
	system($aln);	
	$trigger = $sr;
        $trigger =~ s/PIRSR/SRHMM/;
        $triggerFile = $tmpDir."/".$pirsf."/sr/aln/".$trigger.".aln";
	getTrigger($tmpDir, $pirsf, $sr, $searchOutFile, $alnOutFile, $templateAC, $trigger, $triggerFile);
	getGFF($tmpDir, $pirsf, $seqFile, $triggerFile); 
}

$hasMatch = 0;
open(GFFOUT, ">" , $gffFile) or die "Can't open $gffFile\n";
for my $key (sort keys %gff) {
	$hasMatch = 1;
	print GFFOUT $gff{$key};	
}
close(GFFOUT);

if($hasMatch == 0) {
	print "No match found!\n";
}
sub getGFF {
	($outDir, $pirsf, $seqFile, $triggerFile) = @_;
	%seqLen = ();
	open(SEQ, $seqFile) or die "Can't open $seqFile\n";
	while($line=<SEQ>) {
                chomp($line);
                if ($line =~ /^\>/)  {
                	($Id) = (split(/\s+/, $line))[0];
                         $Id =~ s/^\>//;
                }
                else    {
                        $seqLen{$Id} += length($line) if $Id;
                }
	}
	close(SEQ);	
	open(TRIGGER, $triggerFile) or die "Can't open $triggerFile\n";
	while($line=<TRIGGER>) {
		chomp($line);
		($trigger, $ac, $feature) = (split(/\t/, $line))[0, 1, 2];
		$gffRecord = $ac."\t"."PIRSR"."\t".$trigger."\t"."1"."\t".$seqLen{$ac}."\t"."0"."\t"."."."\t"."."."\t"."Selected 1"."\t".$feature."\n";
		$gff{$ac."-".$trigger} = $gffRecord;	
	}	
	close(TRIGGER);	
}

sub getTrigger {
	($outDir, $pirsf, $sr, $searchOutFile, $alnOutFile, $templateAC) = @_;  	
	%e = ();
	%n = ();	
        open(K, $searchOutFile) or die "Can't open $searchOutFile\n";
	while($line=<K>) {
        	if ($line=~/Sequence\s+Description$/) {
               		$d=1;
                	$line=<K>;
        	}
        	elsif (length($line)<2) {
                	$d=0;
        	}
        	elsif ($d) {
                	$line =~ s/^\s+//;
                	($b, $a)=(split /\s+/,$line)[0,8];
			if($a < 0.0001) {
                		$e{$a}=1;
			}
        	}
	}
	close K;
	open(IN,$alnOutFile) or die "Can't open $alnOutFile\n";
        while($line=<IN>) {
                chomp $line;
                if ($line=~/^\#/) {
                        $x=1;
                }
                elsif (length($line)<5) {
                        $x=1;
                }
                else {
                        ($a,$b)=(split /\s+/,$line)[0,1];
                        $b=~s/\./\-/g; $b=uc($b);
                        $n{$a}.=$b;
                }
        }
        close IN;
	
	open(TRIGGER, ">", $triggerFile) or die "Can't open $triggerFile\n";
        foreach $x (sort keys %n) {
                if ($e{$x} && $x !~ /\-t/){
                        print TRIGGER "$trigger\t$x\tSequence \"$n{$x}\" ; FeatureTemplate \"$templateAC\" \"$n{$templateAC}\" \;\n";
                }
        }
        close(TRIGGER);

}


sub prepareOutDir {
	($outDir, $pirsf) = @_;
	$seqDir  = $outDir."/".$pirsf."/hmm/seq";
	if( -d $seqDir) {
		system("rm -rf $seqDir");
		system("mkdir -p $seqDir");	
	}
	else {
		system("mkdir -p $seqDir");	
	}
	$alnDir = $outDir."/".$pirsf."/hmm/aln";
	if( -d $alnDir) {
		system("rm -rf $alnDir");
		system("mkdir -p $alnDir");	
	}
	else {
		system("mkdir -p $alnDir");	
	}
	$searchDir = $outDir."/".$pirsf."/hmm/search";
	if( -d $searchDir) {
		system("rm -rf $0searchDir");
		system("mkdir -p $searchDir");	
	}
	else {
		system("mkdir -p $searchDir");	
	}
	$srAlnDir = $outDir."/".$pirsf."/sr/aln";
        if( -d $srAlnDir) {
                system("rm -rf $srAlnDir");
                system("mkdir -p $srAlnDir");
        }
        else {
                system("mkdir -p $srAlnDir");
        }

}

sub getQuerySeq {
	($querySeqFile) = @_;
	$seq = "";
	open(QUERY, $querySeqFile) or die "Can't open $querySeqFile\n";
	while($line=<QUERY>) {
  		if ($line=~/^\>/)  {
			chomp($line);
			$seq .= (split(/\s+/, $line))[0]."\n";
		}
		else {
			$seq .= $line;
		}
	}	
	close(QUERY);
	return $seq;
}

sub getTemplateSeqs {
	($srTpSeqFile) = @_;
	my %tpSeq = ();
	open(SRTP, $srTpSeqFile) or die "Can't open $srTpSeqFile\n";
	while($line=<SRTP>) {
  		chomp $line;
  		if ($line=~/^\>/)  {
  			if ($tpSeq{$sr}){
				$tpSeq{$sr}.="\n";
			} 
  			($ac,$sr)=(split /\t/,$line)[0,1]; 
  			$tpSeq{$sr}="$ac\n";
  		}
  		else {
			$tpSeq{$sr} .=$line;
  		} 
	}
	close(SRTP);
	return %tpSeq;	
}

sub getSRIds {
	($srHmmDir, $pirsf) = @_;
	@SRs = ();
	$pirsr = $pirsf;
	$pirsr =~ s/PIRSF/PIRSR/;
 
	opendir(SRHMM, $srHmmDir) or die $!;
	$hasRule = 0;	
	while($file = readdir(SRHMM)) {
		if($file =~ /$pirsr/) {
			$file =~ s/\.hmm$//;
			push(@SRs, $file);
			$hasRule = 1;
		}
	}	
	close(SRHMM);	
	if($hasRule == 0) {
		print "There are no PIR Site Rules for $pirsf!\n";
		exit 1;
	}		
	return @SRs;	
}

sub checkConfFile {
	open(CONF, $conf) or die "Can't open $conf!\n";
	while($line=<CONF>) {
		if($line !~ /^#/ && $line !~ /^$/) {
			chomp($line);
			($key, $value) = (split(/\=/, $line))[0, 1];
			if($key eq "hmmSearchCmd") {
				$hmmSearchCmd = $value;
			}		
			elsif($key eq "hmmAlignCmd") {
				$hmmAlignCmd = $value;
			}		
			elsif($key eq "srHmmDir") {
				$srHmmDir = $value;
				$srHmmDir =~ s/\/$//;
			}		
			elsif($key eq "srTpSeq") {
				$srTpSeq = $value;
				$srTpSeq =~ s/\/$//;
			}
			elsif($key eq "tmpDir") {
				$tmpDir = $value;
				$tmpDir =~ s/\/$//;
			}
			elsif($key eq "debug") {
				$debug = $value;
			}
			else {
				print "Invalid entry \"$key\" in $conf file!\n";
				exit 1;
			}		
		}
	}
	close(CONF);

	if($hmmSearchCmd eq "")  {
		print "You need to specify the path to hmmer3 hmmsearch command in the $conf file!\n";
		exit 1;	
	}
	if($hmmAlignCmd eq "")  {
		print "You need to specify the path to hmmer3 hmmalign command in the $conf file!\n";
		exit 1;	
	}
	if($srHmmDir eq "")  {
		print "You need to specify the path to the directory of site rule hmm info in the $conf file!\n";
		exit 1;	
	}
	if($srTpSeq eq "")  {
		print "You need to specify the path to the site rule template sequence in the $conf file!\n";
		exit 1;	
	}
}



